@@ -11,6 +11,8 @@
 #
 from distutils.command.config import config
 
+from matplotlib.backends.backend_wx import fake_stderr
+
 from ..utils import utils
 from .csip_access import csip_worker
 from pyswarms.single.global_best import GlobalBestPSO
@@ -24,35 +26,54 @@ import queue
 import threading
 
 
-def eval_cost(x, step_param_names, step_objfunc, calib_params, req_queue, files, url, param):
+def eval_cost(x, step_param_names, step_objfunc, calib_params, req_queue, files, url, param, conf: Dict, r, s):
     no_particles = len(x[:, 0])
-    cost = np.ones(no_particles)
-    res_queue = queue.Queue()
 
-    print('   ', end='', flush=True)
+    pfail_count = conf.get('particles_fail', 1)   # Number of particles allowed to fail.
+    pfail_retry = conf.get('particles_retry', 3)  # retry number of times if more than allowed fail
 
-    # submit for processing
-    # for i_particle, v in enumerate(x[:, 0]):
-    for i_particle in range(no_particles):
-        req_queue.put((i_particle, x, step_param_names, calib_params, step_objfunc, res_queue))
-        # req_queue.put((i_particle, x[i_particle,:], step_param_names, calib_params, step_objfunc, res_queue))
+    while pfail_retry > 0:
+        cost = np.ones(no_particles)
+        res_queue = queue.Queue()
 
-    # wait for the cost value to come back
-    # for i, v in enumerate(x[:, 0]):
-    for i in range(no_particles):
-        (i_particle, p_cost) = res_queue.get()
-        cost[i_particle] = p_cost
-        res_queue.task_done()
+        print('   ', end='', flush=True)
 
-    res_queue.join()
+        # submit for processing
+        # for i_particle, v in enumerate(x[:, 0]):
+        for i_particle in range(no_particles):
+            req_queue.put((r, s, i_particle, x, step_param_names, calib_params, step_objfunc, res_queue))
 
-    # replace the 'nan' cost values (failed/missing runs) with the mean of the
-    # rest of the cost values, hence ignore it
+            # req_queue.put((i_particle, x[i_particle,:], step_param_names, calib_params, step_objfunc, res_queue))
 
-    print("cost ", cost)
+        # wait for the cost value to come back
+        # for i, v in enumerate(x[:, 0]):
+        for i in range(no_particles):
+            (i_particle, p_cost) = res_queue.get()
+            cost[i_particle] = p_cost
+            res_queue.task_done()
+
+        res_queue.join()
+
+        # replace the 'nan' cost values (failed/missing runs) with the mean of the
+        # rest of the cost values, hence ignore it
+
+        # print("cost ", cost)
+        nan_idx = np.where(np.isnan(cost))
+        failed_particles = len(nan_idx[0])
+
+        # leave the loop if fails acceptable
+        if failed_particles <= pfail_count:
+            break
+
+        pfail_retry -= 1
+
+    if pfail_retry == 0:
+        print('Particle evaluation failed ', conf.get('particles_retry', 3), ' times. PSO stopped.')
+        return None
+
+    # print("mean ", mean)
+    # assign the mean value to all failed runs.
     mean = np.nanmean(cost)
-    print("mean ", mean)
-    nan_idx = np.where(np.isnan(cost))
     cost[nan_idx[0]] = mean
 
     print(flush=True)
@@ -165,6 +186,7 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
             args['calib_params'] = utils.get_calibrated_params(steps, s)
 
             args['req_queue'] = req_queue
+            args['conf'] = conf
 
             # create optimizer in the first round.
             if optimizer[s] is None:
@@ -173,8 +195,13 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
             print('\n>>>>> R{}/S{}  particle params: {}  calibrated params: {}\n'.format(r + 1, s + 1, param_names,
                                                                                          args['calib_params']))
 
+            args['r'] = r+1
+            args['s'] = s+1
+
             # perform optimization
             cost, pos = optimizer[s].optimize(eval_cost, iters=iters, **args)
+            if cost is None:
+                exit(1)
 
             # capture the best cost
             # if cost < best_cost[s] and np.abs(cost - best_cost[s]) > rtol:
