@@ -29,9 +29,6 @@ def calc_cost(response: "Client", objfunc) -> float:
     """aggregated objective function value -> cost"""
 
     # this will eliminate the failed run as a candidate solution.
-    print(response.is_finished())
-    print(response.http_status)
-    print(response.get_metainfo('status'))
     if not response.is_finished():
         return np.nan
 
